package com.uoc.tellmetweet.bo;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.uoc.tellmetweet.dao.ActiveLocationDAO;
import com.uoc.tellmetweet.dao.ListsDAO;
import com.uoc.tellmetweet.dao.TrendDAO;
import com.uoc.tellmetweet.domain.TmActivelocations;
import com.uoc.tellmetweet.domain.TmTrend;
import com.uoc.tellmetweet.dto.ActiveLocationDTO;
import com.uoc.tellmetweet.dto.ChartDTO;
import com.uoc.tellmetweet.dto.FilterTrendDTO;
import com.uoc.tellmetweet.dto.TrendDTO;
import com.uoc.tellmetweet.exception.TellmetweetBOException;
import com.uoc.tellmetweet.utils.TellmetweetUtils;

@Component
@Transactional
public class ListBO {
	
	private static Logger logger = LoggerFactory.getLogger(ListBO.class);

	@Autowired
	TrendDAO trendDAO;
	
	@Autowired
	ActiveLocationDAO activeLocationDAO;
	
	@Autowired
	ListsDAO listsDAO; 

	
	/**
	 * Obtiene el listado de los trendtopics pertenecientes a un Woeid
	 * @param woeid
	 * @return List<TrendDTO>
	 * @throws TellmetweetBOException
	 */
	public List<TrendDTO> findAllHomeTrendsByWoeid(FilterTrendDTO filterTrendDTO, int page, int limit, int start) throws TellmetweetBOException {
		
		List<TrendDTO> trendDTOList = new ArrayList<TrendDTO>();
		try {
			List<TmTrend> tmTrendList = trendDAO.findAllTrendByWoeid(filterTrendDTO, page, limit, start);
			for (TmTrend tmTrend : tmTrendList) {
				TrendDTO trendDTO = new TrendDTO();
				trendDTO.setName(tmTrend.getName());
				trendDTO.setLocationName(tmTrend.getTrends().getLocation().getName());
				trendDTO.setFcreate(TellmetweetUtils.calendarToDate(tmTrend.getFcreate()));
				trendDTOList.add(trendDTO);
			}
			
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new TellmetweetBOException(e.getMessage(), e);
		}
		return trendDTOList;
	}
	/* end findHomeTrendsByWoeid */
	
	
	/**
	 * Obtiene el número de HomeTrends by woeid
	 * @param filterTrendDTO
	 * @return int
	 */
	public int countFindAllHomeTrendsByWoeid(FilterTrendDTO filterTrendDTO) throws TellmetweetBOException {
		return trendDAO.countFindAllTrendByWoeid(filterTrendDTO);
	}
	/* end countFindAllHomeTrendsByWoeid */
	
	/**
	 * Obtiene el listado de ActiveLocations
	 * @return List<ActiveLocationDTO>
	 * @throws TellmetweetBOException
	 */
	public List<ActiveLocationDTO> findAllActiveLocation() throws TellmetweetBOException {
		
		List<ActiveLocationDTO> activeLocationDTOList = new ArrayList<ActiveLocationDTO>();
		try {
			List<TmActivelocations> tmActivelocationsList = activeLocationDAO.findAllActiveLocations();
			for (TmActivelocations tmActivelocations : tmActivelocationsList) {
				ActiveLocationDTO activeLocationDTO = new ActiveLocationDTO();
				activeLocationDTO.setLocationName(tmActivelocations.getName());
				activeLocationDTO.setWoeid(tmActivelocations.getLocation().getWoeid());
				activeLocationDTO.setLatitude(tmActivelocations.getLatitude().floatValue());
				activeLocationDTO.setLongitude(tmActivelocations.getLongitude().floatValue());
				activeLocationDTOList.add(activeLocationDTO);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new TellmetweetBOException(e.getMessage(), e);
		}
		return activeLocationDTOList;
	}
	/* end findAllActiveLocation */
	
	
	/**
	 * Obtiene la lista de trends para mostrarlos en la gráfica
	 * @param filterTrendDTO
	 * @return List<ChartDTO>
	 * @throws TellmetweetBOException
	 */
	public List<ChartDTO> findAllChartByWoeid(FilterTrendDTO filterTrendDTO) throws TellmetweetBOException {
		
		List<ChartDTO> listChartDTO = new ArrayList<ChartDTO>();
		try {
			List<Object[]> objlist = listsDAO.findAllChartByWoeid(filterTrendDTO);
			for (Object[] obj : objlist) {
				ChartDTO chartDTO = new ChartDTO();
				chartDTO.setName((String) obj[0]);
				chartDTO.setDay(((BigInteger) obj[1]).intValue());
				if(obj[2]!=null){
					chartDTO.setWeek(((BigInteger) obj[2]).intValue());
				}
				if(obj[3]!=null){
					chartDTO.setMonth(((BigInteger) obj[3]).intValue());
				}
				listChartDTO.add(chartDTO);
			}

		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new TellmetweetBOException(e.getMessage(), e);
		}
		return listChartDTO;
	}
	/* end findAllChartByWoeid */
}
