package com.uoc.tellmetweet.bo;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.uoc.tellmetweet.dao.ActiveLocationDAO;
import com.uoc.tellmetweet.dao.LocationsDAO;
import com.uoc.tellmetweet.dao.TrendsDAO;
import com.uoc.tellmetweet.dao.TwitterUtilsDAO;
import com.uoc.tellmetweet.domain.TmActivelocations;
import com.uoc.tellmetweet.domain.TmTrend;
import com.uoc.tellmetweet.domain.TmTrends;
import com.uoc.tellmetweet.dto.HomeTrendsDTO;
import com.uoc.tellmetweet.dto.MessageDTO;
import com.uoc.tellmetweet.exception.TellmetweetBOException;
import com.uoc.tellmetweet.utils.TellmetweetUtils;

import twitter4j.Location;
import twitter4j.ResponseList;
import twitter4j.Trend;
import twitter4j.Trends;
import twitter4j.TwitterFactory;

@Component
@Transactional
public class TrendsBO {
	
	private static Logger logger = LoggerFactory.getLogger(TrendsBO.class);

	@Autowired
	TwitterUtilsDAO twitterUtilsDAO;
	
	@Autowired
	LocationsDAO locationsDAO;
	
	@Autowired
	TrendsDAO trendsDAO;

	@Autowired
	ActiveLocationDAO activeLocationDAO; 
	
	
	/**
	 * storeTrends
	 */
	public List<MessageDTO> storeTrends() throws TellmetweetBOException {

		List<MessageDTO> listMessageDTO = new ArrayList<MessageDTO>(); 
		MessageDTO messageDTO = new MessageDTO();
		try {
			long t1 = System.currentTimeMillis();
			logger.info("Iniciando proceso storeTrends");
			
			/* 1- Establecer la conexión con el API de Twitter obteniendo un TwitterFactory*/
			TwitterFactory tf = twitterUtilsDAO.getTweeterFactoryConnection();
			
			/* 2- Obtener la lista de Locations */
			ResponseList<Location> listLocations = locationsDAO.findAllLocations(tf);
			
			/* 3- Insertar el location en la BD si no existe */
			locationsDAO.saveAllLocations(listLocations);
		
			
			/* 4- Recorrer la tm_activeLocations */
			List<TmActivelocations> listActiveLocation = activeLocationDAO.findAllActiveLocations();  /* Lista de locations activas para usar en la aplicación */
			if(listActiveLocation==null || listActiveLocation.isEmpty()){
				messageDTO = new MessageDTO();
				messageDTO.setDescripcion("No hay locations activas");
				listMessageDTO.add(messageDTO);
			}
			
			// Obtener la lista de trends para el location
			for (TmActivelocations tmActivelocation : listActiveLocation) {
				Trends trends = trendsDAO.findAllTrendsByLocation(tf, tmActivelocation);
				TmTrends tmTrends = new TmTrends();
				tmTrends.setAccesslevel(trends.getAccessLevel());
				tmTrends.setAsof(TellmetweetUtils.dateToCalendar(trends.getAsOf()));
				tmTrends.setFcreate(TellmetweetUtils.dateToCalendar(new Date()));
				tmTrends.setLocation(tmActivelocation.getLocation());
				tmTrends.setTrendat(TellmetweetUtils.dateToCalendar(trends.getTrendAt()));
				
				Set<TmTrend> listTmTrend = new HashSet<TmTrend>();
				for(Trend trend : trends.getTrends()) {
					TmTrend tmTrend = new TmTrend();
					tmTrend.setFcreate(TellmetweetUtils.dateToCalendar(new Date()));
					tmTrend.setName(trend.getName());
					tmTrend.setQuery(trend.getQuery());
					tmTrend.setUrl(trend.getURL());
					listTmTrend.add(tmTrend);
				}
				tmTrends.setTmTrends(listTmTrend);
				
				// Insertar Trends en BD
				trendsDAO.saveTmTrends(tmTrends);
			}
			logger.info("Fin proceso storeTrends");
			long t2 = System.currentTimeMillis();
			messageDTO = new MessageDTO();
			long t = t2 - t1;
			messageDTO.setDescripcion("Trends recuperados en " + TellmetweetUtils.longToDateFormat(t));
			listMessageDTO.add(messageDTO);
			
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new TellmetweetBOException(e.getMessage(), e);
		}
		return listMessageDTO;
	}
	/* end storeTrends */
	
	
	/**
	 * Devuelve los trends de un woeid
	 * @param woeid
	 * @return List<HomeTrendsDTO> 
	 */
	public List<HomeTrendsDTO> findHomeTrendsByWoeid(Integer woeid) throws TellmetweetBOException {
		
		List<HomeTrendsDTO> listHomeTrendsDTO = new ArrayList<HomeTrendsDTO>();
		try {
			
			List<TmTrend> trendsList = trendsDAO.findAllHomeTrend(woeid);
			for (TmTrend tmTrend : trendsList) {
				HomeTrendsDTO homeTrendsDTO = new HomeTrendsDTO();
				homeTrendsDTO.setName(tmTrend.getName());
				homeTrendsDTO.setLocationName(tmTrend.getTrends().getLocation().getName());
				homeTrendsDTO.setWoeid(tmTrend.getTrends().getLocation().getWoeid());
				listHomeTrendsDTO.add(homeTrendsDTO);
			}
			
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new TellmetweetBOException(e.getMessage(), e);
		}
		return listHomeTrendsDTO;
	}
	/* end findHomeTrendsByWoeid */
}
