package com.uoc.tellmetweet.bo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.uoc.tellmetweet.dao.UserLocationDAO;
import com.uoc.tellmetweet.domain.TmUserlocations;
import com.uoc.tellmetweet.dto.UserLocationDTO;
import com.uoc.tellmetweet.exception.TellmetweetBOException;
import com.uoc.tellmetweet.utils.TellmetweetUtils;

@Component
@Transactional
public class UserLocationBO {
	
	private static Logger logger = LoggerFactory.getLogger(UserLocationBO.class);
	
	@Autowired
	UserLocationDAO userLocationDAO;
	
	/**
	 * Devuelve las userLocations de un usuario
	 * @param userEmail
	 * @return UserLocationDTO
	 * @throws TellmetweetBOException
	 */
	public UserLocationDTO getUserLocation(String userEmail) throws TellmetweetBOException {
		
		UserLocationDTO userLocationDTO = new UserLocationDTO();
		try {
			TmUserlocations tmUserlocations = userLocationDAO.findTmUsersLocations(userEmail);
			if(tmUserlocations!=null){
				userLocationDTO.setWoeid1(tmUserlocations.getWoeid1());
				userLocationDTO.setWoeid2(tmUserlocations.getWoeid2());
				userLocationDTO.setWoeid3(tmUserlocations.getWoeid3());
				if(tmUserlocations.getFupdate()!=null){
					userLocationDTO.setFupdate(TellmetweetUtils.calendarToDate(tmUserlocations.getFupdate()));
				}
			}

		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new TellmetweetBOException(e.getMessage(), e);
		}
		
		return userLocationDTO;
	}
	
	
	/**
	 * Actualiza un TmUserlocations
	 * @param userEmail
	 * @param userLocationDTO
	 * @return UserLocationDTO
	 * @throws TellmetweetBOException
	 */
	public Integer update(String userEmail, UserLocationDTO userLocationDTO) throws TellmetweetBOException {
		
		int n = 0;
		try {
			TmUserlocations tmUserlocations = userLocationDAO.findTmUsersLocations(userEmail);
			if(tmUserlocations!=null){
				tmUserlocations.setWoeid1(userLocationDTO.getWoeid1());
				tmUserlocations.setWoeid2(userLocationDTO.getWoeid2());
				tmUserlocations.setWoeid3(userLocationDTO.getWoeid3());
				tmUserlocations.setFupdate(TellmetweetUtils.dateToCalendar(new java.util.Date()));
				n = userLocationDAO.update(tmUserlocations);
			}

		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw new TellmetweetBOException(e.getMessage(), e);
		}
		
		return n;
	}
	
	
	
}
