package com.uoc.tellmetweet.dao;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.uoc.tellmetweet.common.GenericDAOImpl;
import com.uoc.tellmetweet.domain.TmActivelocations;
import com.uoc.tellmetweet.exception.TellmetweetDAOException;

@Transactional
@Repository
public class ActiveLocationDAOImpl extends GenericDAOImpl implements ActiveLocationDAO {
	
	private static Logger logger = LoggerFactory.getLogger(ActiveLocationDAOImpl.class);
	
    @PersistenceContext
    EntityManager entityManager;
    
	/**
	 * Devuelve un listado de tmActivelocations
	 * @return tmActivelocations
	 */
	public List<TmActivelocations> findAllActiveLocations() throws TellmetweetDAOException {
		
		List<TmActivelocations> tmActivelocations = new ArrayList<TmActivelocations>(); 
		
		try {
			tmActivelocations = TmActivelocations.findAllTmActivelocationses();
		} catch (Exception e) {
			logger.error("Error: " + e.getMessage(), e);
			throw new TellmetweetDAOException(e.getMessage(), e);
		}
		return tmActivelocations;
	}
	/* end findAllActiveLocations */
	

	/**
	 * Devuelve un activeLocation pasando un woeid
	 * @param woeid
	 * @return TmActivelocations
	 */
    public TmActivelocations findActiveLocationsByWoeid(Integer woeid) throws TellmetweetDAOException {
    	logger.info("findActiveLocationsByWoeid");
        return entityManager.createQuery("SELECT o FROM TmActivelocations o where o.location.woeid=:woeid", TmActivelocations.class)
        		.setParameter("woeid", woeid)
        		.getSingleResult();
    }
    /* end findActiveLocationsByWoeid */
}
