package com.uoc.tellmetweet.dao;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import com.uoc.tellmetweet.common.GenericDAOImpl;
import com.uoc.tellmetweet.domain.TmTrends;
import com.uoc.tellmetweet.dto.FilterTrendDTO;
import com.uoc.tellmetweet.exception.TellmetweetDAOException;

@Repository
public class ListsDAOImpl extends GenericDAOImpl implements ListsDAO {
	
	private static Logger logger = LoggerFactory.getLogger(ListsDAO.class);

	
	/**
	 * Obtiene la lista de trends para mostrarlos en la gráfica
	 * 
	 * @param filterTrendDTO
	 * @param page
	 * @param limit
	 * @param start
	 * 
	 * @return List<Object[]>
	 * @throws TellmetweetDAOException
	 */
	@SuppressWarnings("unchecked")
	public List<Object[]> findAllChartByWoeid(FilterTrendDTO filterTrendDTO) throws TellmetweetDAOException {
		
		try {
			
			StringBuilder dayHistogramQuery = new StringBuilder();
			dayHistogramQuery.append(" select tema.name, count(tema.name)");
			dayHistogramQuery.append(" from tm_trend tema");
			dayHistogramQuery.append(" inner join tm_trends temas on tema.trends = temas.id");
			dayHistogramQuery.append(" inner join tm_locations location on location.id = temas.location");
			dayHistogramQuery.append(" where DAY(DATE(tema.fcreate)) = DAY(DATE(:from_date))");
			dayHistogramQuery.append(" and WEEK(DATE(tema.fcreate)) = WEEK(DATE(:from_date))");
			dayHistogramQuery.append(" and MONTH(DATE(tema.fcreate)) = MONTH(DATE(:from_date))");
			dayHistogramQuery.append(" and location.woeid = :woeid");
			dayHistogramQuery.append(" group by tema.name");
			dayHistogramQuery.append(" order by tema.name");
			
			StringBuilder weekHistogramQuery = new StringBuilder();
			weekHistogramQuery.append(" select tema.name, count(tema.name)");
			weekHistogramQuery.append(" from tm_trend tema");
			weekHistogramQuery.append(" inner join tm_trends temas on tema.trends = temas.id");
			weekHistogramQuery.append(" inner join tm_locations location on location.id = temas.location");
			weekHistogramQuery.append(" where WEEK(DATE(tema.fcreate)) = WEEK(DATE(:from_date)) and location.woeid = :woeid");
			weekHistogramQuery.append(" and MONTH(DATE(tema.fcreate)) = MONTH(DATE(:from_date))");
			weekHistogramQuery.append(" group by tema.name");
			
			StringBuilder monthHistogramQuery = new StringBuilder();
			monthHistogramQuery.append(" select tema.name, count(tema.name)");
			monthHistogramQuery.append(" from tm_trend tema");
			monthHistogramQuery.append(" inner join tm_trends temas on tema.trends = temas.id");
			monthHistogramQuery.append(" inner join tm_locations location on location.id = temas.location");
			monthHistogramQuery.append(" where MONTH(DATE(tema.fcreate)) = MONTH(DATE(:from_date)) and location.woeid = :woeid");
			monthHistogramQuery.append(" group by tema.name");
			
			List<Object[]> dayHistogram = TmTrends.entityManager().createNativeQuery(dayHistogramQuery.toString())
					.setParameter("woeid", filterTrendDTO.getWoeid())
					.setParameter("from_date", filterTrendDTO.getFrom_date())
					.getResultList();
			
			Map<String, BigInteger> weekHistogram = histogramToMap(TmTrends.entityManager().createNativeQuery(weekHistogramQuery.toString())
					.setParameter("woeid", filterTrendDTO.getWoeid())
					.setParameter("from_date", filterTrendDTO.getFrom_date())
					.getResultList());
			
			Map<String, BigInteger> monthHistogram = histogramToMap(TmTrends.entityManager().createNativeQuery(monthHistogramQuery.toString())
					.setParameter("woeid", filterTrendDTO.getWoeid())
					.setParameter("from_date", filterTrendDTO.getFrom_date())
					.getResultList());

			List<Object[]> result = new ArrayList<Object[]>();
			for (Object[] objects : dayHistogram) {
				result.add(new Object[] { objects[0], objects[1], weekHistogram.get(objects[0]), monthHistogram.get(objects[0]) });
			}
			
			return result;			
			
		} catch (Exception e) {
			logger.error("Error: " + e.getMessage(), e);
			throw new TellmetweetDAOException(e.getMessage(), e);
		}
		
	}
	/* end findAllChartByWoeid */

	private Map<String, BigInteger> histogramToMap(List<Object[]> resultList) throws TellmetweetDAOException {
		Map<String, BigInteger> map = new HashMap<String, BigInteger>();
		for (Object[] row : resultList) {
			map.put((String) row[0], (BigInteger) row[1]);			
		}
		return map;
	}
	
	
}
