package com.uoc.tellmetweet.dao;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.uoc.tellmetweet.common.GenericDAOImpl;
import com.uoc.tellmetweet.common.TellmetweetConstants;
import com.uoc.tellmetweet.domain.TmActivelocations;
import com.uoc.tellmetweet.domain.TmTrend;
import com.uoc.tellmetweet.domain.TmTrends;
import com.uoc.tellmetweet.exception.TellmetweetDAOException;

import twitter4j.Trends;
import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;

@Repository
@Transactional
public class TrendsDAOImpl extends GenericDAOImpl implements TrendsDAO {
	
	private static Logger logger = LoggerFactory.getLogger(TrendsDAO.class);
	
	@Autowired
	TwitterUtilsDAO twitterUtilsDAO;
	
	
	/**
	 * Obtiene la lista de trend de un location 
	 * @param tf - TwitterFactory
	 * @param woeid
	 * @return Trend[]
	 */
	public Trends findAllTrendsByLocation(TwitterFactory tf, TmActivelocations tmActivelocations) throws TellmetweetDAOException {
		
		Trends trends = null;
		try {
			Twitter twitter = tf.getInstance();
			/* Comprobar límite disponible */
			twitterUtilsDAO.checkLimit(tf, TellmetweetConstants.rateLimit_trendsPlace);
			trends = twitter.getPlaceTrends(tmActivelocations.getLocation().getWoeid());
			
		} catch (TwitterException e) {
			logger.error("Error: " + e.getMessage(), e);
			throw new TellmetweetDAOException(e.getMessage(), e);
		}
		
		return trends;
	}
	/* end findAllTrends */
	
	
	/**
	 * Persiste un objeto Trends
	 */
	public void saveTmTrends(TmTrends tmTrends) {
		try {
			save(tmTrends);
			for(TmTrend tmTrend : tmTrends.getTmTrends()) {
				tmTrend.setTrends(tmTrends);
				save(tmTrend);
			}
			
		} catch (Exception e) {
			logger.error("Error: " + e.getMessage(), e);
			throw new TellmetweetDAOException(e.getMessage(), e);
		}
	}
	/* end saveTmTrends */
	

	/**
	 * Obtiene un listado con los últimos trends para mostrarlos en portada
	 * @param listTmActivelocations
	 * @return List<TmTrends>
	 */
	public List<TmTrend> findAllHomeTrend(Integer woeid) throws TellmetweetDAOException {
		
		List<TmTrend> listTmTrend = null;
		try {
			StringBuilder sb = new StringBuilder();
			sb.append("select o from TmTrend o where o.trends.location.woeid=:woeid order by o.id desc");
			listTmTrend = (List<TmTrend>) TmTrends.entityManager().createQuery(sb.toString(), TmTrend.class )
					.setParameter("woeid", woeid)
					.setMaxResults(TellmetweetConstants.TREND_MAX_RESULT)
					.getResultList();	
		} catch (Exception e) {
			logger.error("Error: " + e.getMessage(), e);
			throw new TellmetweetDAOException(e.getMessage(), e);
		}
		
		return listTmTrend;
	}
	/* end findAllHomeTrends */
	
	
	
	
	
}
