package com.uoc.tellmetweet.dao;

import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.uoc.tellmetweet.common.TellmetweetConstants;
import com.uoc.tellmetweet.exception.TellmetweetDAOException;

import twitter4j.Twitter;
import twitter4j.TwitterException;
import twitter4j.TwitterFactory;
import twitter4j.conf.ConfigurationBuilder;

@Repository
@Transactional
public class TwitterUtilsDAOImpl implements TwitterUtilsDAO {
	
	private static Logger logger = LoggerFactory.getLogger(TwitterUtilsDAOImpl.class);

	
	/**
	 * Obtiene la conexión con el API de Twitter
	 * @return twitterFactory
	 */
	public TwitterFactory getTweeterFactoryConnection() throws TellmetweetDAOException {
		if(logger.isDebugEnabled()){logger.debug("TwitterUtilsDAOImpl - getTweeterFactoryConnection");}
		TwitterFactory twitterFactory = new TwitterFactory();
		
    	//Twitter Conf.
		logger.info("Creando configurationBuilder");
		try{
			ConfigurationBuilder cb = new ConfigurationBuilder();
			cb.setDebugEnabled(true)
			.setOAuthConsumerKey(TellmetweetConstants.oauth_consumerKey)
			.setOAuthConsumerSecret(TellmetweetConstants.oauth_consumerSecret)
			.setOAuthAccessToken(TellmetweetConstants.oauth_accessToken)
			.setOAuthAccessTokenSecret(TellmetweetConstants.oauth_accessTokenSecret);
			logger.info("Obteniendo la conexion TwitterFactory");
			twitterFactory = new TwitterFactory(cb.build());
		}catch(Exception e){
			logger.error("Error: " + e.getMessage(), e);
			throw new TellmetweetDAOException(e.getMessage(), e);
		}
		logger.info("Conexión con el API de Twitter obtenida");
		return twitterFactory;
	}
	/* end twitterFactory */
	
	
	/**
	 * Comprueba que no se alcance el límite y queden conexiones disponibles
	 * @param tf
	 * @param rate
	 * @return rateLimit
	 */
	public Integer checkLimit(TwitterFactory tf, String rate) throws TellmetweetDAOException {
		if(logger.isDebugEnabled()){logger.debug("TwitterUtilsDAOImpl - checkLimit");}
		
		Integer rateLimit = 0;
		try {
			Twitter twitter = tf.getInstance();
			
			/* Comprobar límite disponible */
			rateLimit = twitter.getRateLimitStatus().get(rate).getRemaining();
			logger.info("Conexiones disponibles: " + rateLimit + " para el límite " + rate);
			
			while(rateLimit==0) {
				
				Integer secondsUntilReset = twitter.getRateLimitStatus().get(rate).getSecondsUntilReset();
				logger.info("Quedan " + secondsUntilReset + " segundos " + secondsUntilReset/60 +" Minutos - " +  new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(Calendar.getInstance().getTime()));
				
				try {
					Thread.sleep((secondsUntilReset + TellmetweetConstants.rate_wait_marginSeconds) * TellmetweetConstants.rate_wait_milisecondsMultiply);
					rateLimit = twitter.getRateLimitStatus().get(rate).getRemaining();
					if(rateLimit>0){
						logger.info("Se han liberado " + rateLimit + " conexiones para el límite " + rate);
					}
				} catch (InterruptedException e) {
					logger.error("Error: " + e.getMessage(), e);
					throw new TellmetweetDAOException(e.getMessage(), e);
				}
			}
			
			
		} catch (TwitterException e) {
			logger.error("Error: " + e.getMessage(), e);
			throw new TellmetweetDAOException(e.getMessage(), e);
		}
		 
		 return rateLimit;
	}
	/* end checkLimit */ 
	
}
