package com.uoc.tellmetweet.dao;

import com.uoc.tellmetweet.domain.TmUsers;
import com.uoc.tellmetweet.exception.TellmetweetBOException;
import com.uoc.tellmetweet.exception.TellmetweetDAOException;

public interface UserDAO {

    /**
     * Obtiene el usuario logueado
     * @param email
     * @param pass
     * @return TmUsers
     * @throws TellmetweetDAOException
     */
    public TmUsers loginUser(String email, String pass) throws TellmetweetDAOException;
    
    /**
     * Crea un nuevo usuario
     * @param tmUsers
     * @return Integer
     * @throws TellmetweetDAOException
     */
    public Integer registerUser(TmUsers tmUsers) throws TellmetweetDAOException;
    
    /**
     * Comprueba las querys realizadas en el día actual
     * @param userEmail
     * @return Boolean
     * @throws TellmetweetBOException
     */
    public Integer querysUsed(String userEmail) throws TellmetweetBOException;
    
    /**
     * Suma una query al usuario
     * @param userEmail
     * @throws TellmetweetBOException
     */
    public void sumOneQuery(String userEmail) throws TellmetweetBOException;
    
    /**
     * Obtiene el número de querys dependiendo del rol del usuario
     * @param userEmail
     * @return
     * @throws TellmetweetBOException
     */
    public Integer querysAvailableByRol(String userEmail) throws TellmetweetBOException;
    
    /**
     * Obtiene un TmUsers por email
     * @param email
     * @return
     */
    public TmUsers findTmUsersByEmail(String email) throws TellmetweetBOException;
    
	/**
	 * Actualiza un objeto TmUsers en la base de datos
	 */
	public void updateUser(TmUsers tmUsers) throws TellmetweetDAOException;
}
