package com.uoc.tellmetweet.dao;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.uoc.tellmetweet.common.GenericDAOImpl;
import com.uoc.tellmetweet.domain.TmUserlocations;
import com.uoc.tellmetweet.exception.TellmetweetBOException;
import com.uoc.tellmetweet.exception.TellmetweetDAOException;

@Transactional
@Repository
public class UserLocationDAOImpl extends GenericDAOImpl implements UserLocationDAO {

	private static Logger logger = LoggerFactory.getLogger(UserLocationDAOImpl.class);
	
    @PersistenceContext
    EntityManager entityManager;
    
    /**
     * Crea un nuevo userLocation
     * @param tmUserlocations
     * @return Integer
     * @throws TellmetweetDAOException
     */
    public Integer create(TmUserlocations tmUserlocations) throws TellmetweetDAOException {
    	
    	int n=0;
    	try {
    		tmUserlocations.persist();
    		n = 1;
    	} catch (Exception e) {
			logger.error("Error: " + e.getMessage(), e);
    	}
        return n;
    }
    
    
    /**
     * Devuelve las userLocations de un usuario
     * @param email
     * @return TmUserlocations
     * @throws TellmetweetBOException
     */
    public TmUserlocations findTmUsersLocations(String email) throws TellmetweetBOException {
 	   
    	TmUserlocations tmUserlocations = new TmUserlocations();
 		try {
 			StringBuilder sb = new StringBuilder();
 			sb.append("select o from TmUserlocations o where o.cduser.email=:email");
 			List<TmUserlocations> result = (List<TmUserlocations>) TmUserlocations.entityManager().createQuery(sb.toString())
 					.setParameter("email", email)
 					.setMaxResults(1)
 					.getResultList();
 			
 			if(result!=null && result.size()>0){
 				tmUserlocations = result.get(0);
 			}
 			
 		} catch (Exception e) {
 			logger.error("Error: " + e.getMessage(), e);
 			throw new TellmetweetDAOException(e.getMessage(), e);
 		}
 		return tmUserlocations;
    }
    
    
    /**
     * Actualiza un TmUserlocations
     * @param tmUserlocations
     * @return int
     * @throws TellmetweetDAOException
     */
    public Integer update(TmUserlocations tmUserlocations) throws TellmetweetDAOException {
    	
    	int n=0;
    	try {
    		tmUserlocations.merge();
    		n = 1;
    	} catch (Exception e) {
			logger.error("Error: " + e.getMessage(), e);
    	}
        return n;
    }
	
}
