package com.uoc.tellmetweet.utils;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.uoc.tellmetweet.common.TellmetweetConstants;

public class TellmetweetUtils {
	
	private static Logger logger = LoggerFactory.getLogger(TellmetweetUtils.class);
	
	public Calendar getCurrentCalendar() {
		Calendar cal = Calendar.getInstance();
		return cal;
	}

	public static Calendar dateToCalendar(Date date){ 
		  Calendar cal = Calendar.getInstance();
		  cal.setTime(date);
		  return cal;
	}
	
	public static Date calendarToDate(Calendar cal){
		return cal.getTime();
	}
	
	public static Date stringToDate(String dateString) {
		SimpleDateFormat sdf = new SimpleDateFormat(TellmetweetConstants.FORMAT_DATE);
		Date date = null;
		try {
			date= sdf.parse(dateString);
			String stringDate = sdf.format(date);
			date = sdf.parse(stringDate);
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return date;
	}
	
	public static Calendar stringToCalendar(String dateString) {
		DateFormat df = new SimpleDateFormat(TellmetweetConstants.FORMAT_DATE);
		Calendar cal  = Calendar.getInstance();
		try {
			cal.setTime(df.parse(dateString));
		} catch (ParseException e) {
			e.printStackTrace();
		}
		return cal;
	}
	
	public static Date getCalendarDateWithoutTime(Date date) {
	    Calendar cal = Calendar.getInstance();
	    cal.setTime(date);
	    cal.set(Calendar.HOUR_OF_DAY, 0);
	    cal.set(Calendar.MINUTE, 0);
	    cal.set(Calendar.SECOND, 0);
	    cal.set(Calendar.MILLISECOND, 0);
	    return cal.getTime();
	}
	
	public static String longToDateFormat(Long t){
		final Calendar cal = Calendar.getInstance();
		cal.setTimeInMillis(t);
		return new SimpleDateFormat(TellmetweetConstants.FORMAT_TIME).format(cal.getTime());
	}
	
	public static String generateToken() {
		StringBuilder builder = new StringBuilder();
		SecureRandom rand = null;
		try {
			rand = SecureRandom.getInstance("SHA1PRNG");
		} catch (NoSuchAlgorithmException e) {
			return "";
		}
		byte bytes[] = new byte[30];
		rand.nextBytes(bytes);
		for (int i = 0; i < bytes.length; i++) {
			builder.append(String.format("%02X", bytes[i]));
		}
		return builder.toString();
	}
	
	public static String  passwordToMd5(String pass){
		MessageDigest md;
		StringBuilder sb = new StringBuilder();
		try {
			md = MessageDigest.getInstance("MD5");
			md.update(pass.getBytes());
			byte[] bytes = md.digest();
			for(int i=0; i< bytes.length ;i++){
				sb.append(Integer.toString((bytes[i] & 0xff) + 0x100, 16).substring(1));
			}
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return sb.toString();
	}
	
}
