package com.uoc.tellmetweet.web;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uoc.tellmetweet.bo.ListBO;
import com.uoc.tellmetweet.bo.UserBO;
import com.uoc.tellmetweet.dto.ActiveLocationDTO;
import com.uoc.tellmetweet.dto.ChartDTO;
import com.uoc.tellmetweet.dto.FilterDTO;
import com.uoc.tellmetweet.dto.FilterTrendDTO;
import com.uoc.tellmetweet.dto.TrendDTO;
import com.uoc.tellmetweet.dto.WhereDTO;
import com.uoc.tellmetweet.exception.TellmetweetBOException;
import com.uoc.tellmetweet.utils.TellmetweetUtils;

@Controller
@RequestMapping("/rest/list")
public class ListController {
	
	private static Logger logger = LoggerFactory.getLogger(ListController.class);
	
	@Autowired
	ListBO listBO;
	
	@Autowired
	UserBO userBO;
	

	/**
	 * Obtiene el listado de los trendtopics pertenecientes a un Woeid
	 * @param woeid
	 * @return TrendDTO
	 */
	@RequestMapping(value="/trendList",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> listTrends(@RequestParam(required=false) String filterJson,
			@RequestParam(required=false, defaultValue="0") Integer page,
			@RequestParam(required=false, defaultValue="1") Integer start,
			@RequestParam(required=false, defaultValue="0") Integer limit, 
			@RequestParam(required=false) String userEmail, HttpServletRequest request) {
		if(logger.isDebugEnabled()){logger.debug("ListController - listTrends");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<TrendDTO> trendDTOList = new ArrayList<TrendDTO>();
		try {
			
			// permisos
			if(userEmail==null || userEmail.trim().isEmpty()){
				return new ResponseEntity<String>("Only allowed for registered users",headers, HttpStatus.INTERNAL_SERVER_ERROR);
			}
			if(!userBO.hashPermission(userEmail)){
				return new ResponseEntity<String>("Your daily limit has expired",headers, HttpStatus.INTERNAL_SERVER_ERROR);
			}
			
			FilterTrendDTO filterTrendDTO = new FilterTrendDTO();
			
			if((null != filterJson) && (filterJson.length() > 2)){
				FilterDTO filterDTO = new FilterDTO();
				filterDTO = filterDTO.fromJsonToFilterDTO(filterJson);
				
				if((null != filterDTO.getWhere()) && (filterDTO.getWhere().size() > 0)) {
					for(WhereDTO iterWhere : filterDTO.getWhere()){
						if("woeid".equals(iterWhere.getName())){
							filterTrendDTO.setWoeid((Integer) iterWhere.getValue().get(0));
						}
						if("from_date".equals(iterWhere.getName())){
							if(iterWhere.getValue()!=null && iterWhere.getValue().get(0)!=null){
								filterTrendDTO.setFrom_date(TellmetweetUtils.stringToDate((String) iterWhere.getValue().get(0)));
							}
						}
						if("to_date".equals(iterWhere.getName())){
							if(iterWhere.getValue()!=null && iterWhere.getValue().get(0)!=null){
								filterTrendDTO.setTo_date(TellmetweetUtils.stringToDate((String) iterWhere.getValue().get(0)));
							}
						}
					}
				}
				
			trendDTOList = listBO.findAllHomeTrendsByWoeid(filterTrendDTO, page, limit ,start);
			}
			
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>(TrendDTO.toJsonArray(trendDTOList), headers, HttpStatus.OK);
	}
	
	
	/**
	 * Obtiene el listado de ActiveLocations
	 * @return List<ActiveLocationDTO>
	 */
	@RequestMapping(value="/activeLocation",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> listActiveLocations(@RequestParam(required=false) String userEmail) {
		if(logger.isDebugEnabled()){logger.debug("ListController - listActiveLocations");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<ActiveLocationDTO> activeLocationDTOList = new ArrayList<ActiveLocationDTO>();
		try {
			
			activeLocationDTOList = listBO.findAllActiveLocation();
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>(ActiveLocationDTO.toJsonArray(activeLocationDTOList), headers, HttpStatus.OK);
	}
	
	
	@RequestMapping(value="/chart",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> listChart(@RequestParam(required=false) String filterJson, 
			@RequestParam(required=false) String userEmail) {
		if(logger.isDebugEnabled()){logger.debug("ListController - listChart");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<ChartDTO> chartDTOList = new ArrayList<ChartDTO>();
		try {
			
			// permisos
			if(userEmail==null || userEmail.trim().isEmpty()){
				return new ResponseEntity<String>("Only allowed for registered users",headers, HttpStatus.INTERNAL_SERVER_ERROR);
			}
			if(!userBO.hashPermission(userEmail)){
				return new ResponseEntity<String>("Your daily limit has expired",headers, HttpStatus.INTERNAL_SERVER_ERROR);
			}
			
			FilterTrendDTO filterTrendDTO = new FilterTrendDTO();
			
			if((null != filterJson) && (filterJson.length() > 2)){
				FilterDTO filterDTO = new FilterDTO();
				filterDTO = filterDTO.fromJsonToFilterDTO(filterJson);
				
				if((null != filterDTO.getWhere()) && (filterDTO.getWhere().size() > 0)) {
					for(WhereDTO iterWhere : filterDTO.getWhere()){
						if("woeid".equals(iterWhere.getName())){
							filterTrendDTO.setWoeid((Integer) iterWhere.getValue().get(0));
						}
						if("from_date".equals(iterWhere.getName())){
							if(iterWhere.getValue()!=null && iterWhere.getValue().get(0)!=null){
								filterTrendDTO.setFrom_date(TellmetweetUtils.stringToDate((String) iterWhere.getValue().get(0)));
							}
						}
						if("to_date".equals(iterWhere.getName())){
							if(iterWhere.getValue()!=null && iterWhere.getValue().get(0)!=null){
								filterTrendDTO.setTo_date(TellmetweetUtils.stringToDate((String) iterWhere.getValue().get(0)));
							}
						}
					}
				}
				
			chartDTOList = listBO.findAllChartByWoeid(filterTrendDTO);
			}
			
			
			
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>(ChartDTO.toJsonArray(chartDTOList), headers, HttpStatus.OK);
	}
	

}
