package com.uoc.tellmetweet.web;

import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uoc.tellmetweet.bo.TrendsBO;
import com.uoc.tellmetweet.common.Messages;
import com.uoc.tellmetweet.common.TellmetweetConstants;
import com.uoc.tellmetweet.dto.HomeTrendsDTO;
import com.uoc.tellmetweet.dto.MessageDTO;
import com.uoc.tellmetweet.dto.TokenDTO;
import com.uoc.tellmetweet.exception.TellmetweetBOException;

@Controller
@RequestMapping("/rest/trends")
public class TrendsController {
	
	private static Logger logger = LoggerFactory.getLogger(TrendsController.class);
	
	@Autowired
	TrendsBO trendsBO;

	/**
	 * Almacena en la base de datos los últimos Trending Topics, para las localizaciones activas(TM_ACTIVELOCATIONS)
	 * @param request
	 * @return MessageDTO.toJsonArray
	 */
	@RequestMapping(value="/storeTrends",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> listTrends(@RequestBody(required=false) String json){
		if(logger.isDebugEnabled()){logger.debug("TrendsController - storeTrends");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<MessageDTO> listMessageDTO = new ArrayList<MessageDTO>(); 
		TokenDTO tokenDTO = new TokenDTO();
		try {
			
			// seguridad
			if(json!=null && !json.isEmpty()) {
				tokenDTO = TokenDTO.fromJsonToTokenDTO(json);
				if(tokenDTO.getTokenId()==null || !tokenDTO.getTokenId().equals(TellmetweetConstants.security_token)){
					return new ResponseEntity<String>(Messages.usuario_sin_permisos, headers, HttpStatus.INTERNAL_SERVER_ERROR);
				}
			}
			
			logger.info("Iniciando proceso de recuperación de trends");
			listMessageDTO = trendsBO.storeTrends();
			logger.info("Fin proceso de recuperación de trends");
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>(MessageDTO.toJsonArray(listMessageDTO), headers, HttpStatus.OK);
	}
	
	
	/**
	 * Devuelve los últimos Trending Topics almacenados
	 * @param request
	 * @return List<HomeTrendsDTO>
	 */
	@RequestMapping(value="/homeTrends",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> findHomeTrends(@RequestParam(required=false) Integer woeid){
		if(logger.isDebugEnabled()){logger.debug("TrendsController - homeTrends");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<HomeTrendsDTO> listHomeTrendsDTO = null;
		try {
			if(woeid!=null && woeid>0){
				logger.info("Iniciando proceso de recuperación de HomeTrends");
				listHomeTrendsDTO = trendsBO.findHomeTrendsByWoeid(woeid);
				logger.info("Fin proceso de recuperación de HomeTrends");
			}
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		
		return new ResponseEntity<String>(HomeTrendsDTO.toJsonArray(listHomeTrendsDTO), headers, HttpStatus.OK);
	}
}
