package com.uoc.tellmetweet.web;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.paypal.api.payments.Amount;
import com.paypal.api.payments.CreditCard;
import com.paypal.api.payments.FundingInstrument;
import com.paypal.api.payments.Payer;
import com.paypal.api.payments.Payment;
import com.paypal.api.payments.Transaction;
import com.paypal.base.rest.APIContext;
import com.paypal.base.rest.OAuthTokenCredential;
import com.paypal.base.rest.PayPalRESTException;
import com.uoc.tellmetweet.bo.UserBO;
import com.uoc.tellmetweet.common.TellmetweetConstants;
import com.uoc.tellmetweet.dto.CreditCardDTO;
import com.uoc.tellmetweet.dto.UserDTO;
import com.uoc.tellmetweet.exception.TellmetweetBOException;

import twitter4j.JSONObject;

@Controller
@RequestMapping("/rest/user")
public class UserController {
	
	private static Logger logger = LoggerFactory.getLogger(UserController.class);
	
	@Autowired
	UserBO userBO;
	
	

	/**
	 * Comprueba las credenciales de un usuario para loguearse en el sistema
	 * @param json
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/authenticate",headers = "Accept=application/json", method = RequestMethod.POST)
	@ResponseBody
	public ResponseEntity<String> authenticate(@RequestBody String json, HttpServletRequest request) {
		if(logger.isDebugEnabled()){logger.debug("UserController - authenticate");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<UserDTO> listuserDTO = new ArrayList<UserDTO>();
		UserDTO userDTO = new UserDTO();
		try {
		
			if(json!=null){
				JSONObject jsonObj = new JSONObject (json);
				String email = jsonObj.getString("email");
				String pass = jsonObj.getString("pass");
				userDTO = userBO.loginUser(email, pass);
				if(userDTO==null){
					String error ="Credentials incorrect";
					return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ error + "\"}] }", headers, HttpStatus.OK);
				}else{
					listuserDTO.add(userDTO);
				}
			}
			
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ b.getMessage() + "\"}] }", headers, HttpStatus.OK);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(null, headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>("{ \"success\": true, \"data\" : "+ UserDTO.toJsonArray(listuserDTO) +" }", headers, HttpStatus.OK);
	}
	
	
	/**
	 * Crea un nuevo usuario
	 * @param json
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/register",headers = "Accept=application/json", method = RequestMethod.POST)
	@ResponseBody
	public ResponseEntity<String> register(@RequestBody String json, HttpServletRequest request) {
		if(logger.isDebugEnabled()){logger.debug("UserController - authenticate");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<UserDTO> listuserDTO = new ArrayList<UserDTO>();
		UserDTO userDTO = new UserDTO();
		String msg ="";
		try {
		
			if(json!=null){
				userDTO = UserDTO.fromJsonToUserDTO(json);
				
				// Comprobar si ya esite el usuario
				Boolean exist = userBO.existUser(userDTO.getEmail());
				if(exist){
					String error ="The email " + userDTO.getEmail() + " are in use yet";
					return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ error + "\"}] }", headers, HttpStatus.OK);
				}
				
				userDTO = userBO.registerUser(userDTO);
				
				if(userDTO.getPass()!=null){
					msg ="Error creating account";
					return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ msg + "\"}] }", headers, HttpStatus.OK);
				}else{
					listuserDTO.add(userDTO);
				}
			}
		
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ b.getMessage() + "\"}] }", headers, HttpStatus.OK);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(null, headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>("{ \"success\": true, \"data\" : "+ UserDTO.toJsonArray(listuserDTO) +" }", headers, HttpStatus.OK);
	}
	
	
	/**
	 * Realiza el cobro de un usuario Premium
	 * @param json
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/premium",headers = "Accept=application/json", method = RequestMethod.POST)
	@ResponseBody
	public ResponseEntity<String> premium(@RequestBody String json, HttpServletRequest request) {
		if(logger.isDebugEnabled()){logger.debug("UserController - premium");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<CreditCardDTO> listcreditCardDTO = new ArrayList<CreditCardDTO>();
		CreditCardDTO creditCardDTO = new CreditCardDTO();
		List<UserDTO> listuserDTO = new ArrayList<UserDTO>();
		UserDTO userDTO = new UserDTO();
		String msg ="";
		Payment createdPayment = new Payment();
		try {
			if(json!=null){
				
				creditCardDTO = CreditCardDTO.fromJsonToCreditCardDTO(json);
				Map<String, String> sdkConfig = new HashMap<String, String>();
				sdkConfig.put(TellmetweetConstants.CREDIT_CARD_PAYPAL_MODE_LABEL, TellmetweetConstants.CREDIT_CARD_PAYPAL_MODE);
				
				String accessToken = new OAuthTokenCredential(TellmetweetConstants.paypal_id, TellmetweetConstants.paypal_secret, sdkConfig).getAccessToken();
				System.out.println("Token: " + accessToken);
				
				APIContext apiContext = new APIContext(accessToken);
				apiContext.setConfigurationMap(sdkConfig);
				
				CreditCard creditCard = new CreditCard();
				creditCard.setType(creditCardDTO.getType());					// visa
				creditCard.setNumber(creditCardDTO.getNumber());  				// 4020028298038745
				creditCard.setExpireMonth(creditCardDTO.getExpireMonth());		// 01
				creditCard.setExpireYear(creditCardDTO.getExpireYear());		// 2021
				creditCard.setFirstName(creditCardDTO.getFirstName());
				creditCard.setLastName(creditCardDTO.getLastName());
				
				FundingInstrument fundingInstrument = new FundingInstrument();
				fundingInstrument.setCreditCard(creditCard);
				
				List<FundingInstrument> fundingInstrumentList = new ArrayList<FundingInstrument>();
				fundingInstrumentList.add(fundingInstrument);
				
				Payer payer = new Payer();
				payer.setFundingInstruments(fundingInstrumentList);
				payer.setPaymentMethod(TellmetweetConstants.CREDIT_CARD_PAYPAL_PAYMENT_METHOD);
				
				Amount amount = new Amount();
				amount.setCurrency(TellmetweetConstants.CREDIT_CARD_PAYPAL_PAYMENT_CURRENCY);  	// USD
				amount.setTotal(TellmetweetConstants.CREDIT_CARD_PAYPAL_PAYMENT_TOTAL);			// 12
				
				Transaction transaction = new Transaction();
				transaction.setDescription(TellmetweetConstants.CREDIT_CARD_PAYPAL_PAYMENT_DESCRIPTION);
				transaction.setAmount(amount);
				
				List<Transaction> transactions = new ArrayList<Transaction>();
				transactions.add(transaction);
				
				Payment payment = new Payment();
				payment.setIntent(TellmetweetConstants.CREDIT_CARD_PAYPAL_PAYMENT_INTENT);
				payment.setPayer(payer);
				payment.setTransactions(transactions);
				
				try {
					createdPayment = payment.create(apiContext);
				} catch (PayPalRESTException  e) {
					return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ e.getDetails().getDetails().get(0).getIssue() + "\"}] }", headers, HttpStatus.OK);
				}
				
				userDTO = userBO.setPremium(creditCardDTO.getEmail());
			}
		
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ b.getMessage() + "\"}] }", headers, HttpStatus.OK);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(null, headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>("{ \"success\": true, \"data\" : "+ UserDTO.toJsonArray(listuserDTO) +" }", headers, HttpStatus.OK);
	}
	

	/**
	 * Valida un usuario
	 * @param filterJson
	 * @param userEmail
	 * @return
	 */
	@RequestMapping(value="/validate",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> validate(@RequestParam(required=false) String filterJson, 
			@RequestParam(required=false) String userEmail) {
		if(logger.isDebugEnabled()){logger.debug("UserController - validate");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<UserDTO> userDTOList = new ArrayList<UserDTO>();
		try {
			
			if(userEmail!=null && !userEmail.isEmpty()){
				Boolean exist = userBO.existUser(userEmail);
				UserDTO userDTO = new UserDTO();
				if(exist){
					userDTO.setId(-1);
				}
				userDTOList.add(userDTO);
			}
			
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>(UserDTO.toJsonArray(userDTOList), headers, HttpStatus.OK);
	}
	
	
	/**
	 * Obtiene las userLocations
	 * @param filterJson
	 * @param userEmail
	 * @return
	 */
	@RequestMapping(value="/userLocations",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> userLocations(@RequestParam(required=false) String filterJson, 
			@RequestParam(required=false) String userEmail) {
		if(logger.isDebugEnabled()){logger.debug("UserController - validate");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<UserDTO> userDTOList = new ArrayList<UserDTO>();
		try {
			
			if(userEmail!=null && !userEmail.isEmpty()){
				Boolean exist = userBO.existUser(userEmail);
				UserDTO userDTO = new UserDTO();
				if(exist){
					userDTO.setId(-1);
				}
				userDTOList.add(userDTO);
			}
			
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>(UserDTO.toJsonArray(userDTOList), headers, HttpStatus.OK);
	}

}
