package com.uoc.tellmetweet.web;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.uoc.tellmetweet.bo.UserLocationBO;
import com.uoc.tellmetweet.dto.UserLocationDTO;
import com.uoc.tellmetweet.exception.TellmetweetBOException;

import twitter4j.JSONObject;

@Controller
@RequestMapping("/rest/userLocations")
public class UserLocationController {
	
	private static Logger logger = LoggerFactory.getLogger(UserLocationController.class);
	
	@Autowired
	UserLocationBO userLocationBO;
	
	
	
	/**
	 * Obtiene las userLocations
	 * @param filterJson
	 * @param userEmail
	 * @return
	 */
	@RequestMapping(value="",headers = "Accept=application/json", method = RequestMethod.GET)
	@ResponseBody
	public ResponseEntity<String> userLocations(@RequestParam(required=false) String filterJson, 
			@RequestParam(required=false) String userEmail) {
		if(logger.isDebugEnabled()){logger.debug("UserController - validate");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<UserLocationDTO> userLocationDTOList = new ArrayList<UserLocationDTO>();
		try {
			
			if(userEmail!=null && !userEmail.isEmpty()){
				userLocationDTOList.add(userLocationBO.getUserLocation(userEmail));
			}
			
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>(UserLocationDTO.toJsonArray(userLocationDTOList), headers, HttpStatus.OK);
	}
	
	
	/**
	 * Actualizar las userLocations
	 * @param json
	 * @param request
	 * @return
	 */
	@RequestMapping(value="/update",headers = "Accept=application/json", method = RequestMethod.POST)
	@ResponseBody
	public ResponseEntity<String> update(@RequestBody String json, HttpServletRequest request) {
		
		if(logger.isDebugEnabled()){logger.debug("UserController - validate");}
		
		HttpHeaders headers = new HttpHeaders();
		headers.add("Content-Type", "application/json; charset=utf-8");
		
		List<UserLocationDTO> userLocationDTOList = new ArrayList<UserLocationDTO>();
		try {
			
			if(json!=null){
				JSONObject jsonObj = new JSONObject (json);

				UserLocationDTO userLocationDTO = new UserLocationDTO();
				userLocationDTO.setWoeid1(jsonObj.getInt("combo0"));
				userLocationDTO.setWoeid2(jsonObj.getInt("combo1"));
				userLocationDTO.setWoeid3(jsonObj.getInt("combo2"));
				int n = userLocationBO.update(jsonObj.getString("userEmail"), userLocationDTO);
				if(n==0){
					String error ="Can not save";
					return new ResponseEntity<String>("{ \"success\": false , \"errors\" : [ {\"msg\" : \""+ error + "\"}] }", headers, HttpStatus.OK);
				}
				userLocationDTOList.add(userLocationDTO);
			}
			
		} catch (TellmetweetBOException b) {
			logger.error(b.getMessage(), b);
			return new ResponseEntity<String>(b.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return new ResponseEntity<String>(e.getMessage(), headers, HttpStatus.INTERNAL_SERVER_ERROR);
		}
		return new ResponseEntity<String>("{ \"success\": true, \"data\" : "+ UserLocationDTO.toJsonArray(userLocationDTOList) +" }", headers, HttpStatus.OK);
	}

}
