Ext.define('Tellmetweet.view.security.Register', {
    extend: 'Ext.window.Window',
    xtype: 'register',

    requires: [
        'Ext.form.Panel',
        'Tellmetweet.view.security.RegisterController'
    ],
    
    controller	: 'registerController',
    bodyPadding	: 10,
    title		: 'Create new account',
    closable	: false,
    autoShow	: true,
    closeAction	: 'destroy',
    autoScroll	: 'true',
    closable	: true,
    modal		: true,

    items: {
        xtype		: 'form',
        reference	: 'form',
        
        items: [{
            xtype			: 'textfield',
            name			: 'name',
            fieldLabel		: 'Name',
            allowBlank		: false,
            minLength		: 3,
            maxLength		: 32,
            enforceMaxLength: true,
            maskRe			: /^[a-zA-Z\s]+$/,
            listeners		: { afterrender: function(field) {Ext.defer(function() {field.focus(true, 100);}, 1);}}
        },{
            xtype			: 'textfield',
            name			: 'surname1',
            fieldLabel		: 'First Surname',
            allowBlank		: false,
            minLength		: 3,
            maxLength		: 32,
            enforceMaxLength: true,
            vtype			: 'surname', 
            compareTo		: 'name',
            maskRe			: /^[a-zA-Z\s]+$/
        },{
            xtype			: 'textfield',
            name			: 'surname2',
            fieldLabel		: 'Second Surname',
            allowBlank		: false,
            minLength		: 3,
            maxLength		: 32,
            enforceMaxLength: true,
            vtype			: 'surname', 
            compareTo		: 'surname1',
            compareTo2		: 'name',
            maskRe			: /^[a-zA-Z\s]+$/
        },{
            xtype			: 'textfield',
            name			: 'email',
            fieldLabel		: 'Email',
            allowBlank		: false,
            maxLength		: 32,
            vtype			: 'email', 
            enforceMaxLength: true
        },{
            xtype			: 'textfield',
            name			: 'pass',
            inputType		: 'password',
            fieldLabel		: 'Password',
            allowBlank		: false,
            minLength		: 4,
            maxLength		: 32,
            enforceMaxLength: true
        },{
            xtype			: 'textfield',
            inputType		: 'password',
            fieldLabel		: 'Confirm Password',
            allowBlank		: false,
            minLength		: 4,
            maxLength		: 32,
            enforceMaxLength: true,
            vtype			: 'password', 
            comparePass		: 'pass'
        }],
        buttons: [{
            text			: 'Register',
            formBind		: true,
            listeners: {
                click		: 'onRegister'
            }
        }]
    }
});

Ext.onReady(function() {
	
	var me = this;
	
    Ext.apply(Ext.form.field.VTypes, {
        password: function(val, field) {
            var form = field.up('form'); 
            var data = form.getValues();
            var comparePass = data[field.comparePass];
            return val === comparePass;
        },
        passwordText: 'Password do not match',
        	
        surname: function(val, field) {
            var form = field.up('form'); 
            var data = form.getValues();
            var compareTo = data[field.compareTo];
            var compareTo2 = data[field.compareTo2];
            return val !== compareTo && val !== compareTo2;
        },
        surnameText: 'The fields are equals',	
        
        email: function(val, field){
        	var regx = /^(\w+)([\-+.][\w]+)*@(\w[\-\w]*\.){1,5}([A-Za-z]){2,6}$/;
        	return regx.test(val);
        }
        
    });
    

    

    
});

